using System;
using System.Collections;
using System.ComponentModel;
using Gtk;
using Gdk;

namespace WeifenLuo.WinFormsUI.Docking
{
    partial class DockPane
    {
        private class SplitterControl : DrawingArea, ISplitterDragSource
        {
            DockPane m_pane;

            public SplitterControl(DockPane pane)
            {
				// TO-DO: set style
                //SetStyle(ControlStyles.Selectable, false);
                m_pane = pane;
            }

            public DockPane DockPane
            {
                get { return m_pane; }
            }

            private DockAlignment m_alignment;
            public DockAlignment Alignment
            {
                get { return m_alignment; }
                set
                {
                    m_alignment = value;
                    if (m_alignment == DockAlignment.Left || m_alignment == DockAlignment.Right)
						this.GdkWindow.Cursor = new Cursor(CursorType.SbVDoubleArrow);
                    else if (m_alignment == DockAlignment.Top || m_alignment == DockAlignment.Bottom)
						this.GdkWindow.Cursor = new Cursor(CursorType.SbHDoubleArrow);
                    else
						this.GdkWindow.Cursor = new Cursor(CursorType.Arrow);

                    if (DockPane.DockState == DockState.Document)
                        this.QueueDraw();
                }
            }

			protected override bool OnExposeEvent (EventExpose evnt)
			{
                if (DockPane.DockState != DockState.Document)
                    return;

				Gdk.GC g = new Gdk.GC(this.GdkWindow);
				int width, height;
				GdkWindow.GetSize(out width, out height);
				Rectangle rect = new Rectangle(0, 0, width, height);

                if(Alignment == DockAlignment.Top || Alignment == DockAlignment.Bottom)
                    GdkWindow.DrawLine(g, rect.Left, rect.Bottom - 1, rect.Right, rect.Bottom - 1);
                else if (Alignment == DockAlignment.Left || Alignment == DockAlignment.Right)
                    GdkWindow.DrawLine(g, rect.Right - 1, rect.Top, rect.Right - 1, rect.Bottom);

				// TO-DO: unselected color
				ModifyBg(StateType.Insensitive);

				return base.OnExposeEvent (evnt);
			}

			protected override bool OnButtonPressEvent (EventButton evnt)
			{
                if (evnt.Button != 1)
                    return;

				int x, y;
				GdkWindow.GetOrigin(out x, out y);
				int width, height;
				GdkWindow.GetSize(out width, out height);
				Rectangle rect = new Rectangle(x, y, width, height);

                DockPane.DockPanel.BeginDrag(this, rect);

				return base.OnButtonPressEvent (evnt);
			}


            #region ISplitterDragSource Members

            void ISplitterDragSource.BeginDrag(Rectangle rectSplitter)
            {
            }

            void ISplitterDragSource.EndDrag()
            {
            }

            bool ISplitterDragSource.IsVertical
            {
                get
                {
                    NestedDockingStatus status = DockPane.NestedDockingStatus;
                    return (status.DisplayingAlignment == DockAlignment.Left ||
                        status.DisplayingAlignment == DockAlignment.Right);
                }
            }

            Rectangle ISplitterDragSource.DragLimitBounds
            {
                get
                {
                    NestedDockingStatus status = DockPane.NestedDockingStatus;
					int x, y;
					this.GdkWindow.GetOrigin(out x, out y);
					Rectangle rootRect = new Rectangle(x, y, status.LogicalBounds.Width, 
					                                   status.LogicalBounds.Height);
                    Rectangle rectLimit = rootRect;
                    if (((ISplitterDragSource)this).IsVertical)
                    {
                        rectLimit.X += MeasurePane.MinSize;
                        rectLimit.Width -= 2 * MeasurePane.MinSize;
                    }
                    else
                    {
                        rectLimit.Y += MeasurePane.MinSize;
                        rectLimit.Height -= 2 * MeasurePane.MinSize;
                    }

                    return rectLimit;
                }
            }

            void ISplitterDragSource.MoveSplitter(int offset)
            {
                NestedDockingStatus status = DockPane.NestedDockingStatus;
                double proportion = status.Proportion;
                if (status.LogicalBounds.Width <= 0 || status.LogicalBounds.Height <= 0)
                    return;
                else if (status.DisplayingAlignment == DockAlignment.Left)
                    proportion += ((double)offset) / (double)status.LogicalBounds.Width;
                else if (status.DisplayingAlignment == DockAlignment.Right)
                    proportion -= ((double)offset) / (double)status.LogicalBounds.Width;
                else if (status.DisplayingAlignment == DockAlignment.Top)
                    proportion += ((double)offset) / (double)status.LogicalBounds.Height;
                else
                    proportion -= ((double)offset) / (double)status.LogicalBounds.Height;

                DockPane.SetNestedDockingProportion(proportion);
            }

            #region IDragSource Members

            Drawable IDragSource.DragControl
            {
                get { return this; }
            }

            #endregion

            #endregion
        }

        private SplitterControl m_splitter;
        private SplitterControl Splitter
        {
            get { return m_splitter; }
        }

        internal Rectangle SplitterBounds
        {
            set { 
				Splitter.GdkWindow.MoveResize(value);
			}
        }

        internal DockAlignment SplitterAlignment
        {
            set { Splitter.Alignment = value; }
        }
    }
}