using System;
using System.Collections.Generic;
using System.Text;
using Gtk;
using Gdk;

namespace WeifenLuo.WinFormsUI.Docking
{
    internal abstract class InertButtonBase : DrawingArea    {
        protected InertButtonBase()
        {
			// TO-DO set style
            //SetStyle(ControlStyles.SupportsTransparentBackColor, true);
			this.GdkWindow.Opacity = 0;
            //BackColor = Color.Transparent;
        }

        public abstract Gdk.Image Image
        {
            get;
        }

        private bool m_isMouseOver = false;
        protected bool IsMouseOver
        {
            get { return m_isMouseOver; }
            private set
            {
                if (m_isMouseOver == value)
                    return;

                m_isMouseOver = value;
                QueueDraw();
            }
        }

        protected override Size DefaultSize
        {
            get { return new Size(this.Image.Width, this.Image.Height); }
        }

		protected override bool OnMotionNotifyEvent (EventMotion evnt)
		{
            bool over = ClientRectangle.Contains(evnt.X, evnt.Y);
            if (IsMouseOver != over)
                IsMouseOver = over;

			return base.OnMotionNotifyEvent (evnt);
		}

		protected Rectangle ClientRectangle {
			get {
				int width, height;
				this.GdkWindow.GetSize(out width, out height);
				return new Rectangle(0, 0, width, height);

			}
		}

		protected override bool OnExposeEvent (EventExpose evnt)
		{
			Gdk.GC gc = new Gdk.GC(this.GdkWindow);

            if (IsMouseOver)
            {
				GdkWindow.DrawRectangle(gc, false, Rectangle.Inflate(ClientRectangle, -1, -1));
            }

			GdkWindow.DrawImage(gc, Image, 0, 0, 0, 0, this.Image.Width, this.Image.Height);

			return base.OnExposeEvent (evnt);
		}


        public void RefreshChanges()
        {
			int x, y;
			ModifierType type;

			this.GdkWindow.GetPointer(out x, out y, type);

            bool mouseOver = ClientRectangle.Contains(x, y);
            if (mouseOver != IsMouseOver)
                IsMouseOver = mouseOver;

            OnRefreshChanges();
        }

        protected virtual void OnRefreshChanges()
        {
        }
    }
}
